@@ -1,5 +1,5 @@
 /***************************************
-  $Header: /cvsroot/petscgraphics/chts.c,v 1.35 2006/01/30 04:53:58 hazelsct Exp $
+  $Header$
 
   This is the Cahn Hilliard timestepping code.  It is provided here as an
   example usage of the Illuminator Distributed Visualization Library.
@@ -328,7 +328,7 @@ int main (int argc, char **argv)
   else
     ierr = TSSetRHSFunction (thets, ch_ts_residual_vector_2d, &user);
   CHKERRQ(ierr);
-  ierr = TSSetMonitor (thets, ch_ts_monitor, &user, PETSC_NULL); CHKERRQ(ierr);
+  ierr = TSMonitorSet (thets, ch_ts_monitor, &user, PETSC_NULL); CHKERRQ(ierr);
 
   /* This condition prevents the construction of the Jacobian if we're
      running matrix-free. */
@@ -428,7 +428,7 @@ int FormInitialCondition (AppCtx *user, Vec X)
   if (user->random)
     {
       DPRINTF ("Setting initial condition as random numbers\n",0);
-      ierr = PetscRandomCreate (user->comm, RANDOM_DEFAULT, &rand);
+      ierr = PetscRandomCreate (user->comm, &rand);
       CHKERRQ (ierr);
       ierr = PetscRandomSetInterval (rand, 0.35, 0.65); CHKERRQ (ierr);
       ierr = VecSetRandom (X, rand); CHKERRQ (ierr);
