@@ -1,5 +1,5 @@
 /***************************************
-  $Header: /cvsroot/petscgraphics/tsview-ng.c,v 1.65 2006/02/05 21:27:20 hazelsct Exp $
+  $Header$
 
   This program views the output of a time series saved using
   +latex+{\tt IlluMultiSave()}.
@@ -56,18 +56,21 @@ PetscScalar *dp_supp_AB=NULL;
 
 /* Window parameters and drawables */
 #define DEFAULT_RENDER_SIZE 300
-#define SCALE_WIDTH 200
-#define SCALE_HEIGHT 200
 #define SCALE_BPP 3
-int width=0, height=0, bpp=3, nx, ny, transform, dataview_count=1;
+int width=0, height=0, bpp=3, scale_size, nx, ny, transform, dataview_count=1;
 GtkWidget *dataviews [1];
 IDisplay scalar_disp, ternary_square_disp, ternary_triangle_disp, ternary_disp,
   vector_disp, shear_disp;
-gboolean scalar_auto_set=TRUE, ternary_auto_set=TRUE, ternary_square_set=FALSE,
-  vector_auto_set=TRUE, shear_auto_set=TRUE;
+gboolean scalar_auto_set=TRUE, ternary_auto_set=TRUE, vector_auto_set=TRUE,
+  shear_auto_set=TRUE;
 gdouble sizemag;
 PetscTruth transp;
 
+typedef enum {
+  GEN_TRI=0, UNIT_TRI, EQ_TRI, GEN_RECT, UNIT_SQ, SQUARE
+} terntype;
+terntype lastern, thistern;
+
 /* PETSc structures etc. */
 DA theda;
 Vec global;
@@ -106,21 +109,25 @@ int render_dataviews ()
       /* (Re)allocate buffer */
       if (!Disp[0])
 	{
-	  DPRINTF ("Allocating IDisplay RGB buffer\n",0);
+	  DPRINTF ("Allocating IDisplay RGB buffer: %dx%dx%d\n",
+		   width,height,bpp);
 	  if (IDispCreate (Disp, width, height, width, bpp, 0)) {
 	    printf ("ERROR: can't allocate RGB buffer\n");
 	    exit (1); }
 	}
       else
 	{
-	  DPRINTF ("Reallocating IDisplay RGB buffer\n",0);
+	  DPRINTF ("Reallocating IDisplay RGB buffer: %dx%dx%d\n",
+		   width,height,bpp);
 	  if (IDispResize (Disp[0], width, height, width, bpp, 0)) {
 	    printf ("ERROR: can't reallocate RGB buffer\n");
 	    exit (1); }
 	}
-      DPRINTF ("Done, (re)sizing drawing area\n",0);
 
       /* Make sure the drawing area is the right size */
+      DPRINTF ("(Re)sizing drawing area to %dx%d\n",width,height);
+      if (!dataviews [viewnum])
+	dataviews [viewnum] = glade_xml_get_widget (xml, "plot_area");
       gtk_drawing_area_size (GTK_DRAWING_AREA (dataviews[viewnum]),
 			     width, height);
 
@@ -164,16 +171,34 @@ int render_dataviews ()
 
 	    case FIELD_TERNARY:
 	    case FIELD_TERNARY_SQUARE:
-	      if (!ternary_square_set) /* Ternary triangle */
+	      if (thistern==GEN_TRI || thistern==UNIT_TRI || thistern==EQ_TRI)
 		{
 		  GtkWidget *ternary_scale_area = glade_xml_get_widget
 		    (xml, "ternary_scale_area");
 		  int color, point, i;
+		  gboolean update_ternary_entries=FALSE;
 
 		  if (ternary_auto_set)
 		    {
 		      auto_scale (global_array, xm*ym, num_fields,
 				  current_field, FIELD_TERNARY, 2, scales+2);
+		      update_ternary_entries=TRUE;
+		    }
+		  if (thistern==UNIT_TRI)
+		    {
+		      scales[2] = scales[3] = scales[5] = scales[6] = 0.;
+		      scales[4] = scales[7] = 1.;
+		      update_ternary_entries=TRUE;
+		    }
+		  if (thistern==EQ_TRI)
+		    {
+		      scales[5] = scales[3];
+		      scales[6] = scales[2];
+		      scales[7] = scales[3] + scales[4]-scales[2];
+		      update_ternary_entries=TRUE;
+		    }
+		  if (update_ternary_entries)
+		    {
 		      snprintf (maxes[0], 19, "%g", scales[2]);
 		      snprintf (maxes[1], 19, "%g", scales[3]);
 		      snprintf (maxes[2], 19, "%g", scales[4]);
@@ -225,12 +250,28 @@ int render_dataviews ()
 		{
 		  GtkWidget *ternary_scale_area = glade_xml_get_widget
 		    (xml, "ternary_scale_area");
+		  gboolean update_ternary_entries=FALSE;
 
 		  if (ternary_auto_set)
 		    {
 		      auto_scale (global_array, xm*ym, num_fields,
 				  current_field, FIELD_TERNARY_SQUARE, 2,
 				  scales+8);
+		      update_ternary_entries=TRUE;
+		    }
+		  if (thistern==UNIT_SQ)
+		    {
+		      scales[8] = scales[10] = 0.;
+		      scales[9] = scales[11] = 1.;
+		      update_ternary_entries=TRUE;
+		    }
+		  if (thistern==SQUARE)
+		    {
+		      scales[11] = scales[10] + scales[9]-scales[8];
+		      update_ternary_entries=TRUE;
+		    }
+		  if (update_ternary_entries)
+		    {		  
 		      snprintf (maxes[0], 19, "%g", scales[8]);
 		      snprintf (maxes[1], 19, "%g", scales[9]);
 		      snprintf (maxes[2], 19, "%g", scales[10]);
@@ -249,7 +290,7 @@ int render_dataviews ()
 					  maxes[3]);
 		    }
 
-		  render_scale_2d (ternary_triangle_disp, FIELD_TERNARY, 1);
+		  render_scale_2d (ternary_square_disp,FIELD_TERNARY_SQUARE,1);
 		  ierr = render_rgb_local_2d
 		    (Disp[0], global_array, num_fields, current_field,
 		     FIELD_TERNARY_SQUARE, scales+8, nx,ny, xs,ys, xm,ym,
@@ -321,19 +362,17 @@ int render_dataviews ()
       /* Draw IDisplay object onto window */
       DPRINTF ("Painting %dx%d rgb%s buffer onto window\n",
 	       RENDER_WIDTH, RENDER_HEIGHT, (bpp==4) ? "_32" : "");
-      if (!dataviews [viewnum])
-	dataviews [viewnum] = glade_xml_get_widget (xml, "plot_area");
       IDispDrawGdk (Disp[0], dataviews [viewnum], GDK_RGB_DITHER_MAX);
     }
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "myfilter"
-
-/*+ Little variable for myfilter() and refresh_stepnames(). +*/
+/*+ Little variable for my_*_filter() and refresh_stepnames(). +*/
 static char *basefilename;
 
+#undef __FUNCT__
+#define __FUNCT__ "my_time_filter"
+
 /*++++++++++++++++++++++++++++++++++++++
   This function returns non-zero for "qualifying" file names which start with
   the stored files' basename.time and end with
@@ -358,6 +397,9 @@ int my_time_filter (const struct dirent *direntry)
 }
 
 
+#undef __FUNCT__
+#define __FUNCT__ "my_time_filter"
+
 /*++++++++++++++++++++++++++++++++++++++
   This function returns non-zero for "qualifying" file names which start with
   the stored files' basename and end with
@@ -444,7 +486,7 @@ int refresh_stepnames ()
       free (namelist[i]);
       DPRINTF ("[%d] %s\n", i, stepnames [i]);
       if (ierr)
-	printf ("myfilter() Error: %d\n", ierr);
+	printf ("refresh_stepnames() Error: %d\n", ierr);
       /* CHKERRQ (ierr); */
     }
 
@@ -472,6 +514,52 @@ void on_shear_scale_area_expose_event
 (GtkWidget *widget, GdkEventExpose *event, gpointer user_data)
 { IDispDrawGdk (shear_disp, widget, GDK_RGB_DITHER_MAX); }
 
+void on_scale_size_entry_activate (GtkWidget *theentry, gpointer user) {
+  G_CONST_RETURN gchar *entrytext = gtk_entry_get_text (GTK_ENTRY (theentry));
+  guchar black [4] = { 0, 0, 0, 1 };
+  GtkWidget *scalar_scale_area, *ternary_scale_area, *vector_scale_area,
+    *shear_scale_area;
+
+  sscanf (entrytext, "%d", &scale_size);
+
+  if (IDispResize (scalar_disp, scale_size,scale_size,scale_size,SCALE_BPP, 0))
+    { printf ("ERROR: can't reallocate scale RGB buffer\n"); exit (1); }
+  if (IDispResize (ternary_triangle_disp, scale_size,scale_size,scale_size,
+		   SCALE_BPP,0))
+    { printf ("ERROR: can't reallocate scale RGB buffer\n"); exit (1); }
+  if (IDispResize (ternary_square_disp, scale_size,scale_size,scale_size,
+		   SCALE_BPP,0))
+    { printf ("ERROR: can't reallocate scale RGB buffer\n"); exit (1); }
+  if (IDispResize (vector_disp, scale_size,scale_size,scale_size,SCALE_BPP, 0))
+    { printf ("ERROR: can't reallocate scale RGB buffer\n"); exit (1); }
+  if (IDispResize (shear_disp, scale_size,scale_size,scale_size,SCALE_BPP, 0))
+    { printf ("ERROR: can't reallocate scale RGB buffer\n"); exit (1); }
+
+  IDispFill (scalar_disp, black);
+  IDispFill (ternary_disp, black);
+  IDispFill (vector_disp, black);
+  IDispFill (shear_disp, black);
+  render_scale_2d (scalar_disp, FIELD_SCALAR, 1);
+  render_scale_2d (ternary_disp, FIELD_TERNARY, 1);
+  render_scale_2d (vector_disp, FIELD_VECTOR, 1);
+  render_scale_2d (shear_disp, FIELD_TENSOR_SHEAR, 1);
+
+  gtk_drawing_area_size
+    (GTK_DRAWING_AREA (glade_xml_get_widget (xml, "scalar_scale_area")),
+     scale_size, scale_size);
+  gtk_drawing_area_size
+    (GTK_DRAWING_AREA (glade_xml_get_widget (xml, "ternary_scale_area")),
+     scale_size, scale_size);
+  gtk_drawing_area_size
+    (GTK_DRAWING_AREA (glade_xml_get_widget (xml, "vector_scale_area")),
+     scale_size, scale_size);
+  gtk_drawing_area_size
+    (GTK_DRAWING_AREA (glade_xml_get_widget (xml, "shear_scale_area")),
+     scale_size, scale_size);
+
+  render_dataviews ();
+}
+
 void on_scalar_auto_checkbutton_toggled
 (GtkWidget *thebutton, gpointer user_data)
 {
@@ -535,115 +623,218 @@ void on_ternary_auto_checkbutton_toggled
     }
   else
     {
-      gtk_widget_set_sensitive
-	(glade_xml_get_widget (xml, "ternary_1A_label"), TRUE);
-      gtk_widget_set_sensitive
-	(glade_xml_get_widget (xml, "ternary_1B_label"), TRUE);
-      gtk_widget_set_sensitive
-	(glade_xml_get_widget (xml, "ternary_2A_label"), TRUE);
-      gtk_widget_set_sensitive
-	(glade_xml_get_widget (xml, "ternary_2B_label"), TRUE);
-      gtk_widget_set_sensitive
-	(glade_xml_get_widget (xml, "ternary_3A_label"), TRUE);
-      gtk_widget_set_sensitive
-	(glade_xml_get_widget (xml, "ternary_3B_label"), TRUE);
-      gtk_widget_set_sensitive
-	(glade_xml_get_widget (xml, "ternary_1A_entry"), TRUE);
-      gtk_widget_set_sensitive
-	(glade_xml_get_widget (xml, "ternary_1B_entry"), TRUE);
-      gtk_widget_set_sensitive
-	(glade_xml_get_widget (xml, "ternary_2A_entry"), TRUE);
-      gtk_widget_set_sensitive
-	(glade_xml_get_widget (xml, "ternary_2B_entry"), TRUE);
-      gtk_widget_set_sensitive
-	(glade_xml_get_widget (xml, "ternary_3A_entry"), TRUE);
-      gtk_widget_set_sensitive
-	(glade_xml_get_widget (xml, "ternary_3B_entry"), TRUE);
+      if (thistern==GEN_TRI || thistern==EQ_TRI || thistern==GEN_RECT ||
+	  thistern==SQUARE)
+	{
+	  gtk_widget_set_sensitive
+	    (glade_xml_get_widget (xml, "ternary_1A_label"), TRUE);
+	  gtk_widget_set_sensitive
+	    (glade_xml_get_widget (xml, "ternary_1B_label"), TRUE);
+	  gtk_widget_set_sensitive
+	    (glade_xml_get_widget (xml, "ternary_2A_label"), TRUE);
+	  gtk_widget_set_sensitive
+	    (glade_xml_get_widget (xml, "ternary_1A_entry"), TRUE);
+	  gtk_widget_set_sensitive
+	    (glade_xml_get_widget (xml, "ternary_1B_entry"), TRUE);
+	  gtk_widget_set_sensitive
+	    (glade_xml_get_widget (xml, "ternary_2A_entry"), TRUE);
+	}
+      if (thistern==GEN_TRI || thistern==GEN_RECT)
+	{
+	  gtk_widget_set_sensitive
+	    (glade_xml_get_widget (xml, "ternary_2B_label"), TRUE);
+	  gtk_widget_set_sensitive
+	    (glade_xml_get_widget (xml, "ternary_3A_label"), TRUE);
+	  gtk_widget_set_sensitive
+	    (glade_xml_get_widget (xml, "ternary_3B_label"), TRUE);
+	  gtk_widget_set_sensitive
+	    (glade_xml_get_widget (xml, "ternary_2B_entry"), TRUE);
+	  gtk_widget_set_sensitive
+	    (glade_xml_get_widget (xml, "ternary_3A_entry"), TRUE);
+	  gtk_widget_set_sensitive
+	    (glade_xml_get_widget (xml, "ternary_3B_entry"), TRUE);
+	}
     }
 }
 
-void on_ternary_square_checkbutton_toggled
-(GtkWidget *thebutton, gpointer user_data)
+void triangle_to_square ()
 {
-  int field;
   char maxes [6][20];
 
-  ternary_square_set = gtk_toggle_button_get_active
-    (GTK_TOGGLE_BUTTON (thebutton));
+  gtk_widget_hide (glade_xml_get_widget (xml, "ternary_3A_label"));
+  gtk_widget_hide (glade_xml_get_widget (xml, "ternary_3A_entry"));
+  gtk_widget_hide (glade_xml_get_widget (xml, "ternary_3B_label"));
+  gtk_widget_hide (glade_xml_get_widget (xml, "ternary_3B_entry"));
+  gtk_label_set_text (GTK_LABEL (glade_xml_get_widget
+				 (xml, "ternary_1A_label")), "Min A");
+  gtk_label_set_text (GTK_LABEL (glade_xml_get_widget
+				 (xml, "ternary_1B_label")), "Max A");
+  gtk_label_set_text (GTK_LABEL (glade_xml_get_widget
+				 (xml, "ternary_2A_label")), "Min B");
+  gtk_label_set_text (GTK_LABEL (glade_xml_get_widget
+				 (xml, "ternary_2B_label")), "Max B");
+  snprintf (maxes[0], 19, "%g", scales[8]);
+  snprintf (maxes[1], 19, "%g", scales[9]);
+  snprintf (maxes[2], 19, "%g", scales[10]);
+  snprintf (maxes[3], 19, "%g", scales[11]);
+  gtk_entry_set_text (GTK_ENTRY (glade_xml_get_widget
+				 (xml, "ternary_1A_entry")), maxes[0]);
+  gtk_entry_set_text (GTK_ENTRY (glade_xml_get_widget
+				 (xml, "ternary_1B_entry")), maxes[1]);
+  gtk_entry_set_text (GTK_ENTRY (glade_xml_get_widget
+				 (xml, "ternary_2A_entry")), maxes[2]);
+  gtk_entry_set_text (GTK_ENTRY (glade_xml_get_widget 
+				 (xml, "ternary_2B_entry")), maxes[3]);
 
-  for (field=0; field<num_fields; field++)
-    if (fieldtypes [field] == FIELD_TERNARY ||
-	fieldtypes [field] == FIELD_TERNARY_SQUARE)
-      fieldtypes [field] =
-	ternary_square_set ? FIELD_TERNARY_SQUARE : FIELD_TERNARY;
-  render_dataviews ();
+  ternary_disp = ternary_square_disp;
+}
 
-  if (ternary_square_set)
+void square_to_triangle ()
+{
+  char maxes [6][20];
+
+  gtk_widget_show (glade_xml_get_widget (xml, "ternary_3A_label"));
+  gtk_widget_show (glade_xml_get_widget (xml, "ternary_3A_entry"));
+  gtk_widget_show (glade_xml_get_widget (xml, "ternary_3B_label"));
+  gtk_widget_show (glade_xml_get_widget (xml, "ternary_3B_entry"));
+  gtk_label_set_text (GTK_LABEL (glade_xml_get_widget
+				 (xml, "ternary_1A_label")), "Red A");
+  gtk_label_set_text (GTK_LABEL (glade_xml_get_widget
+				 (xml, "ternary_1B_label")), "Red B");
+  gtk_label_set_text (GTK_LABEL (glade_xml_get_widget
+				 (xml, "ternary_2A_label")), "Green A");
+  gtk_label_set_text (GTK_LABEL (glade_xml_get_widget
+				 (xml, "ternary_2B_label")), "Green B");
+  snprintf (maxes[0], 19, "%g", scales[2]);
+  snprintf (maxes[1], 19, "%g", scales[3]);
+  snprintf (maxes[2], 19, "%g", scales[4]);
+  snprintf (maxes[3], 19, "%g", scales[5]);
+  snprintf (maxes[4], 19, "%g", scales[6]);
+  snprintf (maxes[5], 19, "%g", scales[7]);
+  gtk_entry_set_text (GTK_ENTRY (glade_xml_get_widget
+				 (xml, "ternary_1A_entry")), maxes[0]);
+  gtk_entry_set_text (GTK_ENTRY (glade_xml_get_widget
+				 (xml, "ternary_1B_entry")), maxes[1]);
+  gtk_entry_set_text (GTK_ENTRY (glade_xml_get_widget
+				 (xml, "ternary_2A_entry")), maxes[2]);
+  gtk_entry_set_text (GTK_ENTRY (glade_xml_get_widget 
+				 (xml, "ternary_2B_entry")), maxes[3]);
+  gtk_entry_set_text (GTK_ENTRY (glade_xml_get_widget
+				 (xml, "ternary_3A_entry")), maxes[4]);
+  gtk_entry_set_text (GTK_ENTRY (glade_xml_get_widget
+				 (xml, "ternary_3B_entry")), maxes[5]);
+
+  ternary_disp = ternary_triangle_disp;
+}
+
+void change_ternary (GtkWidget *widget, gpointer user_data)
+{
+  GtkWidget *ternary_options, *ternary_menu, *ternary_item, *ternary_items [6];
+  int i, ierr;
+
+  /* Determine the new scale type setting */
+  ternary_options = glade_xml_get_widget (xml, "ternary_scale_menu");
+  ternary_menu = gtk_option_menu_get_menu (GTK_OPTION_MENU (ternary_options));
+  ternary_item = gtk_menu_get_active (GTK_MENU (ternary_menu));
+  ternary_items [0] = glade_xml_get_widget (xml, "general_triangle");
+  ternary_items [1] = glade_xml_get_widget (xml, "unit_triangle");
+  ternary_items [2] = glade_xml_get_widget (xml, "equil_triangle");
+  ternary_items [3] = glade_xml_get_widget (xml, "general_rect");
+  ternary_items [4] = glade_xml_get_widget (xml, "unit_square");
+  ternary_items [5] = glade_xml_get_widget (xml, "square");
+  for (i=0; i<6 && ternary_item != ternary_items[i]; i++) ;
+  if (i==6)
     {
-      gtk_widget_hide (glade_xml_get_widget (xml, "ternary_3A_label"));
-      gtk_widget_hide (glade_xml_get_widget (xml, "ternary_3A_entry"));
-      gtk_widget_hide (glade_xml_get_widget (xml, "ternary_3B_label"));
-      gtk_widget_hide (glade_xml_get_widget (xml, "ternary_3B_entry"));
-      gtk_label_set_text (GTK_LABEL (glade_xml_get_widget
-				     (xml, "ternary_1A_label")), "Min A");
-      gtk_label_set_text (GTK_LABEL (glade_xml_get_widget
-				     (xml, "ternary_1B_label")), "Max B");
-      gtk_label_set_text (GTK_LABEL (glade_xml_get_widget
-				     (xml, "ternary_2A_label")), "Min A");
-      gtk_label_set_text (GTK_LABEL (glade_xml_get_widget
-				     (xml, "ternary_2B_label")), "Max B");
-      snprintf (maxes[0], 19, "%g", scales[8]);
-      snprintf (maxes[1], 19, "%g", scales[9]);
-      snprintf (maxes[2], 19, "%g", scales[10]);
-      snprintf (maxes[3], 19, "%g", scales[11]);
-      gtk_entry_set_text (GTK_ENTRY (glade_xml_get_widget
-				     (xml, "ternary_1A_entry")), maxes[0]);
-      gtk_entry_set_text (GTK_ENTRY (glade_xml_get_widget
-				     (xml, "ternary_1B_entry")), maxes[1]);
-      gtk_entry_set_text (GTK_ENTRY (glade_xml_get_widget
-				     (xml, "ternary_2A_entry")), maxes[2]);
-      gtk_entry_set_text (GTK_ENTRY (glade_xml_get_widget 
-				     (xml, "ternary_2B_entry")), maxes[3]);
-
-      ternary_disp = ternary_square_disp;
+      ierr = PetscPrintf (PETSC_COMM_WORLD, "Unrecognized ternary scale type");
+      exit (1);
     }
-  else
-    {
-      gtk_widget_show (glade_xml_get_widget (xml, "ternary_3A_label"));
-      gtk_widget_show (glade_xml_get_widget (xml, "ternary_3A_entry"));
-      gtk_widget_show (glade_xml_get_widget (xml, "ternary_3B_label"));
-      gtk_widget_show (glade_xml_get_widget (xml, "ternary_3B_entry"));
-      gtk_label_set_text (GTK_LABEL (glade_xml_get_widget
-				     (xml, "ternary_1A_label")), "Red A");
-      gtk_label_set_text (GTK_LABEL (glade_xml_get_widget
-				     (xml, "ternary_1B_label")), "Red B");
-      gtk_label_set_text (GTK_LABEL (glade_xml_get_widget
-				     (xml, "ternary_2A_label")), "Green A");
-      gtk_label_set_text (GTK_LABEL (glade_xml_get_widget
-				     (xml, "ternary_2B_label")), "Green B");
-      snprintf (maxes[0], 19, "%g", scales[2]);
-      snprintf (maxes[1], 19, "%g", scales[3]);
-      snprintf (maxes[2], 19, "%g", scales[4]);
-      snprintf (maxes[3], 19, "%g", scales[5]);
-      snprintf (maxes[4], 19, "%g", scales[6]);
-      snprintf (maxes[5], 19, "%g", scales[7]);
-      gtk_entry_set_text (GTK_ENTRY (glade_xml_get_widget
-				     (xml, "ternary_1A_entry")), maxes[0]);
-      gtk_entry_set_text (GTK_ENTRY (glade_xml_get_widget
-				     (xml, "ternary_1B_entry")), maxes[1]);
-      gtk_entry_set_text (GTK_ENTRY (glade_xml_get_widget
-				     (xml, "ternary_2A_entry")), maxes[2]);
-      gtk_entry_set_text (GTK_ENTRY (glade_xml_get_widget 
-				     (xml, "ternary_2B_entry")), maxes[3]);
-      gtk_entry_set_text (GTK_ENTRY (glade_xml_get_widget
-				     (xml, "ternary_3A_entry")), maxes[4]);
-      gtk_entry_set_text (GTK_ENTRY (glade_xml_get_widget
-				     (xml, "ternary_3B_entry")), maxes[5]);
+  thistern = (terntype) i;
+  DPRINTF ("Changing ternary scale type from %d to %d\n", (int) lastern, i);
 
-      ternary_disp = ternary_triangle_disp;
-    }
+  if ((lastern==GEN_TRI || lastern==UNIT_TRI || lastern==EQ_TRI) &&
+      (thistern==GEN_RECT || thistern==UNIT_SQ || thistern==SQUARE))
+    triangle_to_square ();
+  if ((lastern==GEN_RECT || lastern==UNIT_SQ || lastern==SQUARE) &&
+      (thistern==GEN_TRI || thistern==UNIT_TRI || thistern==EQ_TRI))
+    square_to_triangle ();
   on_ternary_scale_area_expose_event
     (glade_xml_get_widget (xml, "ternary_scale_area"), NULL, NULL);
+
+  /* Sensitize/desensitize the appropriate boxes */
+  if (!ternary_auto_set)
+    {
+      if (thistern==UNIT_TRI || thistern==UNIT_SQ)
+	{
+	  gtk_widget_set_sensitive
+	    (glade_xml_get_widget (xml, "ternary_1A_label"), FALSE);
+	  gtk_widget_set_sensitive
+	    (glade_xml_get_widget (xml, "ternary_1B_label"), FALSE);
+	  gtk_widget_set_sensitive
+	    (glade_xml_get_widget (xml, "ternary_2A_label"), FALSE);
+	  gtk_widget_set_sensitive
+	    (glade_xml_get_widget (xml, "ternary_1A_entry"), FALSE);
+	  gtk_widget_set_sensitive
+	    (glade_xml_get_widget (xml, "ternary_1B_entry"), FALSE);
+	  gtk_widget_set_sensitive
+	    (glade_xml_get_widget (xml, "ternary_2A_entry"), FALSE);
+	}
+      else
+	{
+	  gtk_widget_set_sensitive
+	    (glade_xml_get_widget (xml, "ternary_1A_label"), TRUE);
+	  gtk_widget_set_sensitive
+	    (glade_xml_get_widget (xml, "ternary_1B_label"), TRUE);
+	  gtk_widget_set_sensitive
+	    (glade_xml_get_widget (xml, "ternary_2A_label"), TRUE);
+	  gtk_widget_set_sensitive
+	    (glade_xml_get_widget (xml, "ternary_1A_entry"), TRUE);
+	  gtk_widget_set_sensitive
+	    (glade_xml_get_widget (xml, "ternary_1B_entry"), TRUE);
+	  gtk_widget_set_sensitive
+	    (glade_xml_get_widget (xml, "ternary_2A_entry"), TRUE);
+	}
+      if (thistern==GEN_TRI || thistern==GEN_RECT)
+	{
+	  gtk_widget_set_sensitive
+	    (glade_xml_get_widget (xml, "ternary_2B_label"), TRUE);
+	  gtk_widget_set_sensitive
+	    (glade_xml_get_widget (xml, "ternary_3A_label"), TRUE);
+	  gtk_widget_set_sensitive
+	    (glade_xml_get_widget (xml, "ternary_3B_label"), TRUE);
+	  gtk_widget_set_sensitive
+	    (glade_xml_get_widget (xml, "ternary_2B_entry"), TRUE);
+	  gtk_widget_set_sensitive
+	    (glade_xml_get_widget (xml, "ternary_3A_entry"), TRUE);
+	  gtk_widget_set_sensitive
+	    (glade_xml_get_widget (xml, "ternary_3B_entry"), TRUE);
+	}
+      else
+	{
+	  gtk_widget_set_sensitive
+	    (glade_xml_get_widget (xml, "ternary_2B_label"), FALSE);
+	  gtk_widget_set_sensitive
+	    (glade_xml_get_widget (xml, "ternary_3A_label"), FALSE);
+	  gtk_widget_set_sensitive
+	    (glade_xml_get_widget (xml, "ternary_3B_label"), FALSE);
+	  gtk_widget_set_sensitive
+	    (glade_xml_get_widget (xml, "ternary_2B_entry"), FALSE);
+	  gtk_widget_set_sensitive
+	    (glade_xml_get_widget (xml, "ternary_3A_entry"), FALSE);
+	  gtk_widget_set_sensitive
+	    (glade_xml_get_widget (xml, "ternary_3B_entry"), FALSE);
+	}
+    }
+
+  /* Render the composition field and scale/diffusion path */
+  for (i=0; i<num_fields; i++)
+    if (fieldtypes [i] == FIELD_TERNARY ||
+	fieldtypes [i] == FIELD_TERNARY_SQUARE)
+      fieldtypes [i] =
+	(thistern==GEN_RECT || thistern==UNIT_SQ || thistern==SQUARE) ?
+	FIELD_TERNARY_SQUARE : FIELD_TERNARY;
+  render_dataviews ();
+
+  lastern = thistern;
 }
 
 void on_vector_auto_checkbutton_toggled
@@ -706,19 +897,27 @@ void on_scalar_max_entry_changed (GtkWidget *theentry, gpointer user_data)
 
 void on_ternary_1A_entry_changed (GtkWidget *theentry, gpointer user_data)
 { G_CONST_RETURN gchar *entrytext = gtk_entry_get_text (GTK_ENTRY (theentry));
-  sscanf (entrytext, "%lf", ternary_square_set ? scales+8 : scales+2); }
+  sscanf (entrytext, "%lf",
+	  (thistern==GEN_RECT || thistern==UNIT_SQ || thistern==SQUARE) ?
+	  scales+8 : scales+2); }
 
 void on_ternary_1B_entry_changed (GtkWidget *theentry, gpointer user_data)
 { G_CONST_RETURN gchar *entrytext = gtk_entry_get_text (GTK_ENTRY (theentry));
-  sscanf (entrytext, "%lf", ternary_square_set ? scales+9 : scales+3); }
+  sscanf (entrytext, "%lf",
+	  (thistern==GEN_RECT || thistern==UNIT_SQ || thistern==SQUARE) ?
+	  scales+9 : scales+3); }
 
 void on_ternary_2A_entry_changed (GtkWidget *theentry, gpointer user_data)
 { G_CONST_RETURN gchar *entrytext = gtk_entry_get_text (GTK_ENTRY (theentry));
-  sscanf (entrytext, "%lf", ternary_square_set ? scales+10 : scales+4); }
+  sscanf (entrytext, "%lf",
+	  (thistern==GEN_RECT || thistern==UNIT_SQ || thistern==SQUARE) ?
+	  scales+10 : scales+4); }
 
 void on_ternary_2B_entry_changed (GtkWidget *theentry, gpointer user_data)
 { G_CONST_RETURN gchar *entrytext = gtk_entry_get_text (GTK_ENTRY (theentry));
-  sscanf (entrytext, "%lf", ternary_square_set ? scales+11 : scales+5); }
+  sscanf (entrytext, "%lf",
+	  (thistern==GEN_RECT || thistern==UNIT_SQ || thistern==SQUARE) ?
+	  scales+11 : scales+5); }
 
 void on_ternary_3A_entry_changed (GtkWidget *theentry, gpointer user_data)
 { G_CONST_RETURN gchar *entrytext = gtk_entry_get_text (GTK_ENTRY (theentry));
@@ -883,6 +1082,14 @@ void display_timestep (int usermetacount, char **usermetanames,
 }
 
 
+/*++++++++++++++++++++++++++++++++++++++
+  This function saves both the current field image and also the scale image.
+
+  GtkWidget *widget Standard GTK+ callback argument, ignored here.
+
+  gpointer user_data Standard GTK+ callback argument, ignored here.
+  ++++++++++++++++++++++++++++++++++++++*/
+
 void on_save_activate (GtkWidget *widget, gpointer user_data)
 {
   int i, ierr;
@@ -894,9 +1101,21 @@ void on_save_activate (GtkWidget *widget, gpointer user_data)
   snprintf (number, 9, "-f%d", current_field);
   strncat (filename, number, 198 - strlen (filename));
   strncat (filename, ".ppm", 198 - strlen (filename));
-
-  DPRINTF ("Saving image with filename %s\n", filename);
+  DPRINTF ("Saving field image with filename %s\n", filename);
   IDispWritePPM (Disp[0], filename);
+
+  if (fieldtypes [current_field] == FIELD_TERNARY_SQUARE ||
+      fieldtypes [current_field] == FIELD_TERNARY)
+    {
+      strncpy (filename, basedirname, 198);
+      strcat (filename, "/");
+      strncat (filename, stepnames [entrynum], 198 - strlen (filename));
+      snprintf (number, 9, "-s%d", current_field);
+      strncat (filename, number, 198 - strlen (filename));
+      strncat (filename, ".ppm", 198 - strlen (filename));
+      DPRINTF ("Saving scale image with filename %s\n", filename);
+      IDispWritePPM (ternary_disp, filename);
+    }
 }
 
 
@@ -980,56 +1199,6 @@ void on_save_all_activate (GtkWidget *none, gpointer user_data) {
 }
 
 
-void on_save_all_scale_activate (GtkWidget *none, gpointer user_data) {
-  GtkWidget *timestep_spin = glade_xml_get_widget (xml, "timestep_spin");
-  int i,j, ierr, temp = entrynum;
-  char filename [200], number [10];
-  IDisplay scale_image;
-
-  if (!(scale_image =
-	(fieldtypes [current_field] == FIELD_SCALAR) ? scalar_disp :
-	((fieldtypes [current_field] == FIELD_VECTOR) ? vector_disp :
-	 ((fieldtypes [current_field] == FIELD_TENSOR_SHEAR) ? shear_disp :
-	  ((fieldtypes [current_field] == FIELD_TERNARY_SQUARE ||
-	    fieldtypes [current_field] == FIELD_TERNARY) ? ternary_disp :
-	   NULL)))))
-    return;
-
-  for (i=0; i<total_entries; i++)
-    {
-      gchar appbar_message [30];
-      gtk_spin_button_set_value (GTK_SPIN_BUTTON (timestep_spin), (gfloat) i);
-      on_timestep_spin_value_changed (timestep_spin, user_data);
-
-      sprintf (appbar_message, "Saving image %d/%d", i+1, total_entries);
-      gnome_appbar_set_status
-	(GNOME_APPBAR (glade_xml_get_widget (xml, "main_appbar")),
-	 appbar_message);
-      gnome_appbar_set_progress_percentage
-	(GNOME_APPBAR (glade_xml_get_widget (xml, "main_appbar")),
-	 (gfloat) (i+1)/total_entries);
-      while (gtk_events_pending ())
-	gtk_main_iteration ();
-
-      strncpy (filename, basedirname, 198);
-      strcat (filename, "/");
-      strncat (filename, stepnames [entrynum], 198 - strlen (filename));
-      snprintf (number, 9, "-s%d", current_field);
-      strncat (filename, number, 198 - strlen (filename));
-      strncat (filename, ".ppm", 198 - strlen (filename));
-
-      DPRINTF ("Saving image with filename %s\n", filename);
-      IDispWritePPM (scale_image, filename);
-    }
-
-  gnome_appbar_set_progress_percentage
-    (GNOME_APPBAR (glade_xml_get_widget (xml, "main_appbar")), 0.);
-  gnome_appbar_refresh (GNOME_APPBAR(glade_xml_get_widget(xml,"main_appbar")));
-  gtk_spin_button_set_value (GTK_SPIN_BUTTON (timestep_spin), (gfloat) temp);
-  on_timestep_spin_value_changed (timestep_spin, user_data);
-}
-
-
 void on_refresh_activate (GtkWidget *none, gpointer user_data) {
   if (refresh_stepnames ()) exit (1); }
 
@@ -1108,7 +1277,9 @@ int main (int argc, char *argv[])
 {
   int usermetacount=0, i, ierr;
   char **usermetanames, **usermetadata, filename [200], **field_name;
-  GtkWidget *variable_options, *variable_menu, **variable_item;
+  G_CONST_RETURN gchar *entrytext;
+  GtkWidget *variable_options, *variable_menu, **variable_item,
+    *ternary_options, *ternary_menu, *ternary_item, *ternary_items [6];
 
   /*+ After
     +latex+{\tt PETSc}
@@ -1151,7 +1322,10 @@ int main (int argc, char *argv[])
   strcat (filename, "/tsview.glade");
   DPRINTF ("Loading Glade file %s\n", filename);
   xml = glade_xml_new (filename, NULL, NULL);
+
+  DPRINTF ("Autoconnecting...\n",0);
   glade_xml_signal_autoconnect (xml);
+  DPRINTF ("Done.\n",0);
 
   if (argc>1)
     the_basename = argv[1];
@@ -1237,13 +1411,34 @@ int main (int argc, char *argv[])
   gtk_widget_show (variable_menu);
   gtk_widget_show (variable_options);
 
+  /* Ternary type */
+  ternary_options = glade_xml_get_widget (xml, "ternary_scale_menu");
+  ternary_menu = gtk_option_menu_get_menu (GTK_OPTION_MENU (ternary_options));
+  ternary_item = gtk_menu_get_active (GTK_MENU (ternary_menu));
+  ternary_items [0] = glade_xml_get_widget (xml, "general_triangle");
+  ternary_items [1] = glade_xml_get_widget (xml, "unit_triangle");
+  ternary_items [2] = glade_xml_get_widget (xml, "equil_triangle");
+  ternary_items [3] = glade_xml_get_widget (xml, "general_rect");
+  ternary_items [4] = glade_xml_get_widget (xml, "unit_square");
+  ternary_items [5] = glade_xml_get_widget (xml, "square");
+  for (i=0; i<6 && ternary_item != ternary_items[i]; i++) ;
+  if (i==6)
+    {
+      ierr = PetscPrintf (PETSC_COMM_WORLD, "Unrecognized ternary scale type");
+      exit (1);
+    }
+  lastern = thistern = (terntype) i;
+
   /* Scale images */
-  IDispCreate (&scalar_disp, SCALE_WIDTH,SCALE_HEIGHT,SCALE_WIDTH,SCALE_BPP,0);
-  IDispCreate (&vector_disp, SCALE_WIDTH,SCALE_HEIGHT,SCALE_WIDTH,SCALE_BPP,0);
-  IDispCreate (&shear_disp, SCALE_WIDTH,SCALE_HEIGHT,SCALE_WIDTH,SCALE_BPP,0);
-  IDispCreate (&ternary_triangle_disp, SCALE_WIDTH, SCALE_HEIGHT, SCALE_WIDTH,
+  entrytext = gtk_entry_get_text
+    (GTK_ENTRY (glade_xml_get_widget (xml, "scale_size_entry")));
+  sscanf (entrytext, "%d", &scale_size);
+  IDispCreate (&scalar_disp, scale_size, scale_size, scale_size, SCALE_BPP, 0);
+  IDispCreate (&vector_disp, scale_size, scale_size, scale_size, SCALE_BPP, 0);
+  IDispCreate (&shear_disp, scale_size, scale_size, scale_size, SCALE_BPP, 0);
+  IDispCreate (&ternary_triangle_disp, scale_size, scale_size, scale_size,
 	       SCALE_BPP, 0);
-  IDispCreate (&ternary_square_disp, SCALE_WIDTH, SCALE_HEIGHT, SCALE_WIDTH,
+  IDispCreate (&ternary_square_disp, scale_size, scale_size, scale_size,
 	       SCALE_BPP, 0);
   ternary_disp = ternary_triangle_disp;
   render_scale_2d (scalar_disp, FIELD_SCALAR, 1);
