@@ -1,5 +1,5 @@
 /***************************************
-  $Header: /cvsroot/petscgraphics/tsview.c,v 1.43 2006/01/30 04:53:58 hazelsct Exp $
+  $Header$
 
   This program views the output of a time series saved using
   +latex+{\tt IlluMultiSave()}.
@@ -531,9 +531,11 @@ int main (int argc, char *argv[])
 		break;
 	      }
 
-	    for (entry=0; entry<total_entries; entry++)
+	    for (entry=(instring[1]?atoi(instring+1):0); entry<total_entries;
+		 entry++)
 	      {
-		DPRINTF ("Loading   file %d/%d\r", entry, total_entries);
+		ierr=PetscPrintf
+		  (PETSC_COMM_WORLD, "Loading   file %d/%d\r", entry, total_entries);
 		strcpy (basis, basedirname);
 		strcat (basis, "/");
 		strcat (basis, files[entry]);
@@ -541,7 +543,8 @@ int main (int argc, char *argv[])
 				      &usermetacount, &usermetanames,
 				      &usermetadata); CHKERRQ (ierr);
 
-		DPRINTF ("Rendering file %d/%d\r", entry, total_entries);
+		ierr=PetscPrintf
+		  (PETSC_COMM_WORLD, "Rendering file %d/%d\r", entry, total_entries);
 		ierr = DATriangulateRange
 		  (Surf, theda, global, display_field, minmax,
 		   plots?plots:PETSC_DECIDE, plots?plot_vals:PETSC_NULL,
@@ -554,7 +557,8 @@ int main (int argc, char *argv[])
 		CHKERRQ (ierr);
 		ierr = SurfClear (Surf); CHKERRQ (ierr);
 
-		DPRINTF ("Dumping   file %d/%d\r", entry, total_entries);
+		ierr=PetscPrintf
+		  (PETSC_COMM_WORLD, "Dumping   file %d/%d\r", entry, total_entries);
 		strncpy (filename, basedirname, 198);
 		strcat (filename, "/");
 		strncat (filename, files [entry], 198 - strlen (filename));
@@ -563,7 +567,7 @@ int main (int argc, char *argv[])
 		strncat (filename, ".ppm", 198 - strlen (filename));
 		IDispWritePPM (Disp, filename);
 	      }
-	    DPRINTF ("\n");
+	    DPRINTF ("\n",0);
 	    break;
 	  }
 #endif
